<?php
/**
 * "Платёжная система ВТБ"
 * Контроллер отвечает за генерацию ссылки оплаты и перенаправление
 *
 * @author    VTB <acquiring_support@vtb.ru>
 * @copyright Since 2024 VTB
 */
class vtbpayPaymentModuleFrontController extends ModuleFrontController
{
    /**
     * @var int Идентификатор заказа
     */
    private ?int $order_id = null;


    /**
     * Обработчик POST-запроса.
     *
     * @throws Exception Исключение выбрасывается в случае ошибки.
     * @return void
     */
    public function postProcess()
    {
        $this->module->setVtbpayLogger();
        $this->module->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));
        $this->module->setVtbApi();

        try {
            // Проверяем валидность контекста и доступность способа оплаты
            if (!$this->isValidContext() || !$this->isPaymentOptionAvailable()) {
                throw new \Exception($this->module->l('Валидация не удалась.'));
            }

            if (!Validate::isLoadedObject($this->context->customer)) {
                throw new \Exception($this->module->l('Клиент не найден.'));
            }

            // Инициализируем заказ
            if (!$this->initializeOrder()) {
                throw new \Exception($this->module->l('Заказ не создан.'));
            }

            if (
                !isset($this->module->currentOrder) ||
                empty($this->order_id = $this->module->currentOrder)
            ) {
                throw new \Exception($this->module->l('Идентификатор заказа пуст.'));
            }

            Tools::redirect($this->getPayUrl());

        } catch (\Exception | VtbPayException $e) {
            // Обрабатываем исключение и логируем ошибку
            $this->handleErrorAndRedirect($e);
        }
    }


    /**
     * Инициализация заказа.
     *
     * @return bool Результат инициализации заказа.
     */
    private function initializeOrder()
    {
        $cart = $this->context->cart;
        return $this->module->validateOrder(
            $cart->id,
            Configuration::get('PS_OS_BANKWIRE'),
            (float) $cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            $this->module->l('Заказ инициализирован "Платежной системой ВТБ"'),
            [],
            (int) $cart->id_currency,
            false,
            $cart->secure_key,
            $this->context->shop,
            null
        );
    }


    /**
     * Получение URL для оплаты заказа.
     *
     * @return string URL для оплаты заказа.
     */
    private function getPayUrl()
    {
        $cart = $this->context->cart;
        $order_create = strtotime($cart->date_add);
        $total = $cart->getOrderTotal(true);

        $customer = new Customer($cart->id_customer);
        $email = $customer->email ?: $this->module->configure['VTBPAY_EMAIL_FISCAL'];

        $return_url = $this->context->link->getModuleLink(
            $this->module->name,
            'return', [
                'key' => $cart->secure_key
            ],
            true
        );

        $two_stage = (bool) $this->module->configure['VTBPAY_TWO_STAGE']; // двухстадийная оплата
        $enable_fiscal = (bool) $this->module->configure['VTBPAY_ENABLE_FISCAL'];  // включение фискализации

        $items = [];
        if (!$two_stage && $enable_fiscal) $items = $this->getItems();

        // Логирование входных данных
        $this->module->logger->debug(
            __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderLink - INPUT: ', [
            'arguments' => [
                'order_id' => $this->order_id,
                'email' => $email,
                'order_create' => $order_create,
                'total' => $total,
                'return_url' => $return_url,
                'two_stage' => $two_stage,
                'items' => $items
            ]
        ]);

        $response = $this->module->vtb_api->getOrderLink(
            $this->order_id,
            $email,
            $order_create,
            $total,
            $return_url,
            $two_stage,
            $items
        );

        // Логирование результата
        $this->module->logger->debug(
            __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['payUrl'] ?? '';
    }


    /**
     * Получает массив товаров заказа.
     *
     * @return array Массив товаров.
     */
    private function getItems(): array
    {
        $items = [];
        $key = 1;

        // Настройки по умолчанию для фискальных параметров
        $fiscal_settings = [
            'VTBPAY_MEASURE_FISCAL' => 0,
            'VTBPAY_TAX_TYPE_FISCAL' => 'none',
            'VTBPAY_PAYMENT_TYPE_FISCAL' => 'full_prepayment',
            'VTBPAY_PAYMENT_SUBJECT_FISCAL' => 1
        ];

        // Перебор всех продуктов в корзине
        foreach ($this->context->cart->getProducts() as $item) {

            $product_fiscal = [];
            // Получение фискальных параметров для каждого продукта
            foreach ($fiscal_settings as $name => $default) {
                $product_fiscal[$name] = Configuration::get($name . '_PRODUCT_' . $item['id_product']) ?: '-';
                if ($product_fiscal[$name] === '-') $product_fiscal[$name] = $this->module->configure[$name] ?: $default;
            }

            $items[] = [
                'positionId' => $key,
                'name' => $item['name'],
                'description' => strip_tags($item['description_short']),
                'code' => (string) $item['reference'] ?: $item['id_product'],
                'price' => floatval($item['price']),
                'measure' => (int) $product_fiscal['VTBPAY_MEASURE_FISCAL'],
                'quantity' => (int) $item['cart_quantity'],
                'taxParams' => [
                    'taxType' => $product_fiscal['VTBPAY_TAX_TYPE_FISCAL']
                ],
                'paymentType' => $product_fiscal['VTBPAY_PAYMENT_TYPE_FISCAL'],
                'paymentSubject' => (int) $product_fiscal['VTBPAY_PAYMENT_SUBJECT_FISCAL'],
                'amount' => floatval($item['total']),
                'discount' => floatval($item['reduction'] ?? 0)
            ];

            $key++;
        }

        // Стоимость доставки
        $shipping_cost = $this->context->cart->getOrderTotal(true, Cart::ONLY_SHIPPING); // Стоимость доставки
        if (!empty($shipping_cost)) {
            $shipping_method = new Carrier($this->context->cart->id_carrier);
            $items[] = [
                'positionId' => $key,
                'name' => $shipping_method->name ?: 'Доставка',
                'price' => floatval($shipping_cost),
                'measure' => 0,
                'quantity' => 1,
                'taxParams' => [
                    'taxType' => $this->module->configure['VTBPAY_TAX_TYPE_FISCAL'] ?: 'none'
                ],
                'paymentType' => $this->module->configure['VTBPAY_PAYMENT_TYPE_DELIVERY_FISCAL'] ?: 'full_prepayment',
                'paymentSubject' => 4,
                'amount' => floatval($shipping_cost)
            ];
        }

        return $items;
    }


    /**
     * Проверка, является ли контекст валидным.
     *
     * @return bool Результат проверки валидности контекста.
     */
    private function isValidContext()
    {
        $cart = $this->context->cart;
        return Validate::isLoadedObject($cart)
            && Validate::isUnsignedInt($cart->id_customer)
            && Validate::isUnsignedInt($cart->id_address_delivery)
            && Validate::isUnsignedInt($cart->id_address_invoice);
    }


    /**
     * Убедитесь, что данный способ оплаты по-прежнему доступен.
     *
     * @return bool Результат проверки доступности способа оплаты.
     */
    private function isPaymentOptionAvailable()
    {
        if (!empty($modules = Module::getPaymentModules())) {
            foreach ($modules as $module) {
                if (isset($module['name']) && $this->module->name === $module['name']) {
                    return true;
                }
            }
        }

        return false;
    }


    /**
     * Обработка ошибки и перенаправление.
     *
     * @param Exception|VtbPayException $error_obj Объект ошибки.
     * @return void
     */
    private function handleErrorAndRedirect($error_obj)
    {
        $context = [];
        if (method_exists($error_obj, 'getContext')) $context = $error_obj->getContext();

        $this->module->logger->error(sprintf(
            __CLASS__ . ' > ' . __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
            $error_obj->getMessage(),
            $this->order_id ?? ''
        ), $context);

        $this->errors[] = $error_obj->getMessage();
        $this->redirectWithNotifications($this->context->link->getPageLink(
            'cart',
            true,
            (int) $this->context->language->id,[
                'action' => 'show',
            ]
        ));
    }
}
