<?php
/**
 * Модуль для интеграции с платежной системой ВТБ.
 *
 * @author    VTB <acquiring_support@vtb.ru>
 */

if (!defined('_PS_VERSION_')) exit;

require_once 'vendor/autoload.php';

use PrestaShop\PrestaShop\Core\Payment\PaymentOption,
    \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException,
    \Vtbpay\Form\Modifier\ProductFormModifier;


class VtbPay extends PaymentModule
{
    /**
     * @var array Массив с конфигурацией модуля.
     */
    public array $configure;

    /**
     * @var VtbApi|null Объект для работы с API ВТБ.
     */
    public ?VtbApi $vtb_api = null;

    /**
     * @var VtbPayLogger|null Объект для логирования.
     */
    public VtbPayLogger $logger;


    /**
     * В конструкторе инициализируем настройки модуля.
     */
    public function __construct()
    {
        $this->name = 'vtbpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.4.4';
        $this->author = 'VTB';
        $this->author_uri = 'https://acquiring.vtb.ru';
        $this->logo = __DIR__ . '/videws/img/logotype.png';
        $this->need_instance = 1;

        $this->controllers = [
            'payment',
            'return',
            'webhook'
        ];

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Платежная система ВТБ');
        $this->description = $this->l('Расплачивайтесь любой картой банка без комиссии.');

        $this->confirmUninstall = $this->l('Вы уверены, что хотите удалить этот модуль?');

        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => _PS_VERSION_
        ];

        $this->configure = $this->getConfigFormValues();
    }


    /**
     * Получает объект для работы с API ВТБ.
     *
     * @return VtbApi Объект для работы с API ВТБ.
     */
    public function setVtbApi(): self
    {
        if (empty($this->vtb_api)) {
            $config = $this->configure;
            $this->vtb_api = new VtbApi(
                $config['VTBPAY_CLIENT_ID'],
                $config['VTBPAY_CLIENT_SECRET'],
                (bool) $config['VTBPAY_TEST_MODE'],
                $config['VTBPAY_CLIENT_MERCHANT'] ?: ''
            );
        }

        return $this;
    }


    /**
     * Действия выполняемые при установке.
     *
     * @return bool Результат установки модуля.
     */
    public function install()
    {
        // Checking PHP Version
        if (version_compare(PHP_VERSION, '7.4.0', '<')) {
            $this->_errors[] = $this->l('Для работы платежного модуля "Платежная система ВТБ" требуется PHP версии 7.4.0 или выше.');
            return false;
        }

        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('Для использования этого модуля необходимо включить на сервере расширение cURL.');
            return false;
        }

        Configuration::updateValue('VTBPAY_TEST_MODE', true);

        return parent::install() &&
               $this->registerHook('paymentOptions') &&
               $this->registerHook(['actionProductFormBuilderModifier']) &&
               $this->registerHook(['actionProductSave']);
    }


    /**
     * Действия при удалении модуля.
     *
     * @return bool Результат удаления модуля.
     */
    public function uninstall()
    {
        // Delete module config variable
        Configuration::deleteByName('VTBPAY_TEST_MODE');
        Configuration::deleteByName('VTBPAY_CLIENT_ID');
        Configuration::deleteByName('VTBPAY_CLIENT_SECRET');
        Configuration::deleteByName('VTBPAY_CLIENT_MERCHANT');
        Configuration::deleteByName('VTBPAY_LOGGING');
        Configuration::deleteByName('VTBPAY_TWO_STAGE');
        Configuration::deleteByName('VTBPAY_SUCCESSFUL_PAYMENT_PAGE');
        Configuration::deleteByName('VTBPAY_FAILED_PAYMENT_PAGE');
        Configuration::deleteByName('VTBPAY_ENABLE_FISCAL');
        Configuration::deleteByName('VTBPAY_EMAIL_FISCAL');
        Configuration::deleteByName('VTBPAY_PAYMENT_TYPE_DELIVERY_FISCAL');
        Configuration::deleteByName('VTBPAY_MEASURE_FISCAL');
        Configuration::deleteByName('VTBPAY_TAX_TYPE_FISCAL');
        Configuration::deleteByName('VTBPAY_PAYMENT_TYPE_FISCAL');
        Configuration::deleteByName('VTBPAY_PAYMENT_SUBJECT_FISCAL');

        return parent::uninstall();
    }


    /**
     * Отображает настройки модуля в административной части.
     *
     * @return string HTML-код настроек модуля.
     */
    public function getContent()
    {
        if (((bool)Tools::isSubmit('submitVtbpayModule')) == true) $this->postProcess();

        $this->context->smarty->assign('module_dir', $this->_path);

        return $this->renderForm();
    }


    /**
     * Генерирует форму настроек модуля.
     *
     * @return string HTML-код формы настроек.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitVtbpayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
                                '&configure=' . $this->name .
                                '&tab_module=' . $this->tab .
                                '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }


    /**
     * Возвращает массив с полями настроек.
     *
     * @return array Массив с полями настроек.
     */
    protected function getConfigForm(): array
    {
        return include 'config_form.php';
    }


    /**
     * Возвращает массив значений конфигурации модуля.
     *
     * @return array Массив значений конфигурации.
     */
    protected function getConfigFormValues()
    {
        return [
            'VTBPAY_TEST_MODE' => Configuration::get('VTBPAY_TEST_MODE'),
            'VTBPAY_CLIENT_ID' => Configuration::get('VTBPAY_CLIENT_ID'),
            'VTBPAY_CLIENT_SECRET' => Configuration::get('VTBPAY_CLIENT_SECRET'),
            'VTBPAY_CLIENT_MERCHANT' => Configuration::get('VTBPAY_CLIENT_MERCHANT'),
            'VTBPAY_LOGGING' => Configuration::get('VTBPAY_LOGGING'),
            'VTBPAY_TWO_STAGE' => Configuration::get('VTBPAY_TWO_STAGE'),
            'VTBPAY_SUCCESSFUL_PAYMENT_PAGE' => Configuration::get('VTBPAY_SUCCESSFUL_PAYMENT_PAGE'),
            'VTBPAY_FAILED_PAYMENT_PAGE' => Configuration::get('VTBPAY_FAILED_PAYMENT_PAGE'),
            'VTBPAY_ENABLE_FISCAL' => Configuration::get('VTBPAY_ENABLE_FISCAL'),
            'VTBPAY_EMAIL_FISCAL' => Configuration::get('VTBPAY_EMAIL_FISCAL'),
            'VTBPAY_PAYMENT_TYPE_DELIVERY_FISCAL' => Configuration::get('VTBPAY_PAYMENT_TYPE_DELIVERY_FISCAL'),
            'VTBPAY_MEASURE_FISCAL' => Configuration::get('VTBPAY_MEASURE_FISCAL'),
            'VTBPAY_TAX_TYPE_FISCAL' => Configuration::get('VTBPAY_TAX_TYPE_FISCAL'),
            'VTBPAY_PAYMENT_TYPE_FISCAL' => Configuration::get('VTBPAY_PAYMENT_TYPE_FISCAL'),
            'VTBPAY_PAYMENT_SUBJECT_FISCAL' => Configuration::get('VTBPAY_PAYMENT_SUBJECT_FISCAL')
        ];
    }


    /**
     * Обновляет отправленные настройки модуля.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }


    /**
     * Возвращает опции платежа для отображения на странице оплаты.
     *
     * @param array $params Параметры платежа.
     * @return array Массив опций платежа.
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active || !$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_option = new PaymentOption();
        $payment_option->setModuleName($this->name);
        $payment_option->setCallToActionText($this->l('Платежная система ВТБ'));
        $payment_option->setAction($this->context->link->getModuleLink($this->name, 'payment', [], true));
        $payment_option->setLogo(Media::getMediaPath(__DIR__ . '/views/img/label.png'));

        return [$payment_option];
    }


    /**
     * Добавление кастомных полей на страницу редактирования товара
     *
     * @param array $params Параметры товара.
     * @return void
     */
    public function hookActionProductFormBuilderModifier(array $params): void
    {
        $this->get(ProductFormModifier::class)->modify(
            (int) $params['id'],
            $params['form_builder']
        );
    }


    /**
     * Сохранение значений кастомных полей со страницы редактирования товара
     *
     * @param array $params Параметры товара.
     * @return void
     */
    public function hookActionProductSave(array $params): void
    {
       // We are using configuration table to save the data
       $productData = Tools::getValue('product');
       $idProduct = $params['id_product'];

       $fields = [
           'VTBPAY_MEASURE_FISCAL' => $productData['vtbpay_settings']['VTBPAY_MEASURE_FISCAL'],
           'VTBPAY_TAX_TYPE_FISCAL' => $productData['vtbpay_settings']['VTBPAY_TAX_TYPE_FISCAL'],
           'VTBPAY_PAYMENT_TYPE_FISCAL' => $productData['vtbpay_settings']['VTBPAY_PAYMENT_TYPE_FISCAL'],
           'VTBPAY_PAYMENT_SUBJECT_FISCAL' => $productData['vtbpay_settings']['VTBPAY_PAYMENT_SUBJECT_FISCAL'],
       ];

       foreach ($fields as $key => $value) {
           Configuration::updateValue($key . '_PRODUCT_' . $idProduct, $value);
       }
    }


    /**
     * Проверяет, доступна ли валюта для данного заказа.
     *
     * @param object $cart Объект корзины.
     * @return bool Результат проверки валюты.
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
            return false;
        } else {
            return true;
        }
    }


    /**
     * Изменение статуса платежа в заказе (используется во front контроллерах).
     *
     * @param string $payment_status Статус заказа.
     * @param string $order_id Идентификатор заказа.
     * @return void
     */
    public function changePaymentStatus(
        string $payment_status,
        string $order_id
    ): void {
        $new_order_history = new OrderHistory();
        $new_order_history->id_order = (int) $order_id;
        $new_order_history->id_employee = 0; // ID сотрудника (0 для автоматической записи)

        $available_statuses = [
            'PAID' => Configuration::get('PS_OS_PAYMENT'),
            'PENDING' => Configuration::get('PS_OS_PREPARATION'),
            'REFUNDED' => Configuration::get('PS_OS_REFUND'),
            'VOIDED' => Configuration::get('PS_OS_CANCELED')
        ];
        $order_status = $available_statuses[$payment_status] ?? false;

        $order = new Order((int) $order_id); // Получаем объект заказа
        $current_order_status = $order->getCurrentState(); // Получаем текущий статус заказа

        $new_order_status = $order_status ?: Configuration::get('PS_OS_ERROR');

        if ($current_order_status !== $new_order_status) {
            $new_order_history->changeIdOrderState(
                (int) $new_order_status,
                (int) $order_id
            );

            if (!$new_order_history->addWithemail(true)) {
                throw new \Exception($this->l('Статус платежа не изменился.'));
            }
        }
    }


    /**
     * Устанавливает объект логирования для модуля.
     *
     * @return void
     */
    public function setVtbpayLogger(): void
    {
        $logging = $this->configure['VTBPAY_LOGGING'];

        $file_logger = new FileLogger(PrestaShopLoggerInterface::DEBUG);

        $file_logger->setFilename(dirname(__DIR__, 2) . '/var/logs/vtbpay-' . date('d-m-Y') . '.log');

        $this->logger = VtbPayLogger::getInstance()
                        ->setOption('showCurrentDate', false)
                        ->setOption('showLogLevel', false)
                        ->setCustomRecording(function($message) use ($file_logger)
                        {
                            // Логирование ошибки
                            $file_logger->logError($message);

                            PrestaShopLogger::addLog(
                                explode('Context:', $message)[0],
                                PrestaShopLoggerInterface::ERROR,
                                1,
                                'VtbPay'
                            );

                        }, VtbPayLogger::LOG_LEVEL_ERROR)
                        ->setCustomRecording(function($message) use ($file_logger, $logging)
                        {
                            // Логирование сообщения дебага
                            if ($logging) $file_logger->logDebug($message);

                        }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
